using namespace std;



//overload operator < for sorting SNPs in desending order of supports
bool operator<(const biSNP& a, const biSNP& b) 
{
    return (TOTAL_NUMBER_OF_INDIVIDUALS - a.count()) < (TOTAL_NUMBER_OF_INDIVIDUALS - b.count());
}


//overload operator < for sorting topk_list in decending order of chi_score
bool operator<(const TwoLocus& a, const TwoLocus& b) 
{
    return (0 - a.chi_score) < (0 - b.chi_score);
}

//update topk_list
//keep it within length k
//ordered in decending order of chi_score
/*
void update_topk_list(list<TwoLocus>& topk_list, const TwoLocus& element)
{

	if (topk_list.size()<TOPK) 
	{
		topk_list.push_back(element);
	}

	else 
		if (element.chi_score > topk_list.back().chi_score) 
		{
			if (topk_list.size()==TOPK) 
			{
				topk_list.pop_back();
			}

			topk_list.push_back(element);

		}

		topk_list.sort();
}
*/


void update_topk_list(list<TwoLocus>& topk_list, const TwoLocus& element)
{
    if (topk_list.size()>=TOPK && element.chi_score<=topk_list.back().chi_score)
	return;    
	
    list<TwoLocus>::iterator iter;
    
    list<TwoLocus>::iterator iter_pheno=topk_list.end();
    
    list<TwoLocus>::iterator iter_insert=topk_list.end();
    
    for (iter=topk_list.begin();iter!=topk_list.end();++iter)
    {
	if ((*iter).phenotype_id == element.phenotype_id) 
	{
	    iter_pheno=iter;
	}
	if ( (iter_insert==topk_list.end()) && ((*iter).chi_score<=element.chi_score) )
	{
	    iter_insert=iter;		
	}	    
    }
    
    //not full yet
    if (topk_list.size()<TOPK) 
    {	
	if (iter_pheno!=topk_list.end()) {
	    if ((*iter_pheno).chi_score<element.chi_score){		
		topk_list.insert(iter_insert,element);
		topk_list.erase(iter_pheno);
	    }
	    else{
		//dummy: do nothing.
	    }	    
	}
	else {
	    topk_list.insert(iter_insert,element);
	}	
    }
    //the list is full already
    else 
    {
	if (iter_pheno!=topk_list.end()) {
	    if ((*iter_pheno).chi_score<element.chi_score){		
		topk_list.insert(iter_insert,element);
		topk_list.erase(iter_pheno);
	    }
	    else{
		//dummy: do nothing.
	    }	    
	}
	else {	    
	    topk_list.insert(iter_insert,element);
	    topk_list.pop_back();
	}
    }
    /*
    for (iter=topk_list.begin();iter!=topk_list.end();++iter)
    {
	cout <<(*iter).chi_score <<" ";
	
    }
    cout <<endl;
    */
}


//update topk_list WITH PHENOTYOPE ID
//keep it within length k
//ordered in decending order of chi_score (*changed to the following*)
//the list is unordered, instead, each time scan the list to find the min chi_score in the list when using the upper bound
//the reason is ordering the list is too expensive (10 times more than chi-score calculation)
//theta is the current min chi_score in the list, which is used as decsion factor when the list is full (should the list updated or not)
void update_topk_list_pid(list<TwoLocus>& topk_list, const TwoLocus& element, const float theta)
{


	list<TwoLocus>::iterator iter, miniter; //miniter keep track the position of min chi_score in the list



	//not full yet
	if (topk_list.size()<TOPK) 
	{
//		topk_list.push_back(element);


		for (iter=topk_list.begin();iter!=topk_list.end();iter++)
		{
			if ((*iter).phenotype_id == element.phenotype_id) 
			{
				break;
			}
		}

		//phenotype not exist in the list
		if (iter == topk_list.end()) 
		{
			topk_list.push_back(element);
		}

		//phenotype exists in the list already
		else
		{
			if (element.chi_score > (*iter).chi_score)
			{
				topk_list.erase(iter);

				topk_list.push_back(element);

			}


		}

	}

	//the list is full already
	else 
	{

/*		if (element.F_score > topk_list.back().F_score) 
		{
			if (topk_list.size()==TOPK) 
			{
				topk_list.pop_back();
			}

			topk_list.push_back(element);

		}
*/

		if (element.chi_score > theta)//topk_list.back().chi_score) 
		{

			for (iter=topk_list.begin();iter!=topk_list.end();iter++)
			{
				if ((*iter).phenotype_id == element.phenotype_id) 
				{
					break;
				}
			}

			//phenotype not exist in the list
			if (iter == topk_list.end()) 
			{
				list<TwoLocus>::iterator iterj,

				miniter=topk_list.begin();
				float mm = (*miniter).chi_score;

				for (iterj=topk_list.begin();iterj!=topk_list.end();iterj++)
				{
					if ((*iterj).chi_score < mm) 
					{
						mm = (*iterj).chi_score;
						miniter = iterj;
					}

				}

				topk_list.erase(miniter);

//				topk_list.pop_back();

				topk_list.push_back(element);
			}

			//phenotype exists in the list already
			else
			{

				if (element.chi_score > (*iter).chi_score)
				{
					topk_list.erase(iter);

					topk_list.push_back(element);

				}


			}



		}

	
	}

	//always sort
//	clock_t tt = clock();
//	topk_list.sort();
//	return (clock()-tt);
}

float min_chi_score_in_the_topk_list(list<TwoLocus>& topk_list)
{
	list<TwoLocus>::iterator iter;
	iter = topk_list.begin();
	float mm = (*iter).chi_score;

	for (iter = topk_list.begin();iter!=topk_list.end();iter++)
	{
		if ((*iter).chi_score < mm) 
		{
			mm = (*iter).chi_score;
		}

	}
	return mm;

}
